//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#ifndef DRAWABLE_HPP
#define DRAWABLE_HPP

#include <SFML/Window.hpp>

#include "Object.hpp"
//------------------------------------------------------------------------------
namespace En
{

//------------------------------------------------------------------------------
/// LUA_HEADER_COPY_ON ///
/** A Drawable is a base class which contains fonctions common to every Drawable object (Group, Sprite, Text...). It should not be used directly. */
/// LUA_HEADER_COPY_OFF ///

class Drawable : public Object
{
public:
    Drawable(Group *parent, const std::string &name);
    //Drawable(const Drawable& src);

    /// LUA_EXPORT_ON(Drawable, Object, ProtectedPtr<Object>) ///
    // Zpos
    /** Access the object's z order (an object with a low z order will be in the background) */
    //LUA// luaProperty(int, zPos, GetZpos, SetZpos);

    /** Get the object's z order */
    int GetZpos() const;

    /** Set the object's z order (an object with a low z order will be in the background) */
    void SetZpos(int val);

    /** Get the low pos (the low zPos is a creation order, for the object that have the same zPos) */
    int GetLowZPos() const;

    /** Hide / show the object */
    //LUA// luaProperty(bool, visible, IsVisible, Show);

    /** Show / hide an object */
    void Show(bool visible = true);

    /** Hide and object */
    void Hide();

    /** Return truc if the object is visible, false otherwise */
    bool IsVisible();

    /** Return true if this Drawable is on to of the the given one (higher Zpos) */
    bool IsUpperThan(const Drawable *drawableToCompare) const;

    /// LUA_EXPORT_OFF ///

    // No Lua
    virtual void Render(sf::RenderTarget &App) = 0;
    virtual void ChangeParent(Group *parent);

    // Register clone
    virtual void RegisterClone(Group *newParent);

protected:
    bool m_isVisible;
    int m_zpos;
    int m_lowZpos;
};

class DrawableSF : public Drawable
{
public:
    DrawableSF(Group *parent, const std::string &name);

protected:
    void RenderSf(sf::RenderTarget &App, sf::Drawable *drawable);

};
//------------------------------------------------------------------------------
} // namespace En

#endif // DRAWABLE_HPP
