//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "ScreenRenderer.hpp"
//------------------------------------------------------------------------------
#include "MyException.hpp"
//------------------------------------------------------------------------------
#include "Screen.hpp"
#include "Drawable.hpp"
//------------------------------------------------------------------------------
namespace En
{
using namespace std;
//------------------------------------------------------------------------------
ScreenRenderer::ScreenRenderer(Group *parent, const std::string &name) : Drawable(parent, name), m_targetScreen(NULL), m_screenConfig(Vector2D(1024, 768)), m_isScreenConfigEnabled(false)
{
    //ctor
}
//------------------------------------------------------------------------------
void ScreenRenderer::Render(sf::RenderTarget &App)
{
    const sf::View &backView = App.GetView(); // We need to use a reference as the view is owned by the screen => we have to give it again at the end

    if(m_targetScreen.IsValid())
    {
        Screen *targetScreen = m_targetScreen.GetPtr();

        // We alwaws render without renderScreen to prevent render loop
        /*ProtectedPtr<Screen> renderScreen = targetScreen->GetRenderScreenRaw();
        if(! renderScreen.IsNull())
        {
            if(renderScreen.IsValid())
            {
                targetScreen = renderScreen.GetPtr();
            }
            else
            {
                MY_THROW("ScreenRenderer::Render() : renderScreen of screen '" << targetScreen->GetName() << "' was deleted ! ScreenRenderer name = '" << GetName() << "'.");
            }
        }*/

        if(m_isScreenConfigEnabled)
        {
            ScreenConfig backCfg = targetScreen->GetScreenConfig();
            targetScreen->SetScreenConfig(m_screenConfig);
            targetScreen->RenderNoRenderScreen(App);
            targetScreen->SetScreenConfig(backCfg);
        }
        else
        {
            targetScreen->RenderNoRenderScreen(App);
        }
    }
    else
    {
        MY_THROW("ScreenRenderer::Render() : targetScreen was deleted or is not set ! ScreenRenderer name = '" << GetName() << "'.");
    }

    App.SetView(backView);
}
//------------------------------------------------------------------------------
void ScreenRenderer::SetTargetScreen(ProtectedPtr<Screen> targetScreen)
{
    if(GetParentScreenRaw() == targetScreen)
    {
        MY_THROW("Screen Renderer : a screen can't render itself ! Screen name = '" << targetScreen->GetName()  << "'. ScreenRenderer name = '" << GetName() << "'.");
    }

    m_targetScreen = targetScreen;
}
//------------------------------------------------------------------------------
ProtectedPtr<Screen> ScreenRenderer::GetTargetScreen()
{
    return m_targetScreen;
}
//------------------------------------------------------------------------------
/** Get the target screen config : this screen config will be used to render the target screen */
ScreenConfig ScreenRenderer::GetScreenConfig() const
{
    return m_screenConfig;
}
//------------------------------------------------------------------------------
/** Set the target screen config and enable it : this screen config will be used to render the target screen */
void ScreenRenderer::SetScreenConfig(const ScreenConfig &screenConfig)
{
    m_screenConfig = screenConfig; m_isScreenConfigEnabled = true;
}
//------------------------------------------------------------------------------
/** Return true if the target screen use a special screenConfig (set with SetScreenConfig()), false otherwise. */
bool ScreenRenderer::IsScreenConfigEnabled() const
{
    return m_isScreenConfigEnabled;
}
//------------------------------------------------------------------------------
/** Enable or disable the screenConfig. If it is diabled, the screen will be rendered with its own screen config. */
void ScreenRenderer::EnableScreenConfig(bool isScreenConfigEnabled)
{
    m_isScreenConfigEnabled = isScreenConfigEnabled;
}
//------------------------------------------------------------------------------
} // namespace En
//------------------------------------------------------------------------------
