//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#ifndef SPRITE_HPP
#define SPRITE_HPP

#include "Drawable.hpp"
#include "ClassWithProtectedRc.hpp"

#include <SFML/Graphics.hpp>
//------------------------------------------------------------------------------
namespace En
{
/// LUA_HEADER_COPY_ON ///
/** A Sprite is used to render a texture. It can be resised. To create a Shape, you should use Group::CreateShape. */
/// LUA_HEADER_COPY_OFF ///

class Sprite : public DrawableSF, public sf::Sprite, public ClassWithProtectedRcSingle
{
public:
    Sprite(Group *parent, const std::string &name);
    static Sprite *Create(Group *parent, const std::string &name) {return new Sprite(parent, name);}

    /// LUA_EXPORT_ON(Sprite, Drawable, ProtectedPtr<Object>) ///

    /** Change the sprite size. */
    //LUA// luaProperty(Vector2D, size, GetSize, SetSize);

    /** Get the current size of the displayed sprite */
    Vector2D GetSize() const;

    /** Set the current size of the displayed sprite */
    void SetSize(const Vector2D &newSize);

    /** Set an image that will be drawn by the sprite */
    void SetImage(Texture &image);

    // -------------- From sf::Drawable --------------
    /** Acces object's color. */
    //LUA// luaProperty(Color, color, GetColor, SetColor);
    /** Set object's color. */
    //LUA// void SetColor(const Color& Col);
    /** Get object's color. */
    Color GetColor() const {return sf::Drawable::GetColor();}
    /** Get object's scale */
    Vector2D GetScale() const {return sf::Drawable::GetScale();}
    /** Set object's scale (x) */
    //LUA// void SetScaleX(float FactorX);
    /** Set object's scale (y) */
    //LUA// void SetScaleY(float FactorY);
    /** Set object's scale */
    void SetScale(const Vector2D& scale) {sf::Sprite::SetScale(scale);}
    /** Set object's center */
    void SetCenter(const Vector2D& center) {sf::Sprite::SetCenter(center);}
    /** Get object's center */
    Vector2D GetCenter() const {return sf::Drawable::GetCenter();}

    // From sf::Sprite
    // /!\ //LUA// void SetSubRect(const IntRect& SubRect);
    /** Set if the image should be flipped (X axis) */
    //LUA// void FlipX(bool Flipped);
    /** Set if the image should be flipped (Y axis) */
    //LUA// void FlipY(bool Flipped);
    // /!\ //LUA// const Texture* GetImage() const;
    // /!\ //LUA// const IntRect& GetSubRect() const;
    /** Get the color of a pixel in the image */
    //LUA// Color GetPixel(unsigned int X, unsigned int Y) const;

    /// LUA_EXPORT_OFF ///

    // No Lua
    /** Create a clone of the object, change the parent and return an handler to the clone. */
    virtual Sprite *GetRawCloneToGroup(Group *newParent) const {Sprite *obj = new Sprite(*this); obj->RegisterClone(newParent); return obj;}

    // Acces to position & rotation
    Vector2D GetPosition() const {return DrawableSF::GetPosition();}
    virtual void SetPosition(const Vector2D &pos) {DrawableSF::SetPosition(pos);}
    float GetRotation() const {return DrawableSF::GetRotation();}
    virtual void SetRotation(float angle) {DrawableSF::SetRotation(angle);}
    void Rotate(float angle) {DrawableSF::Rotate(angle);}
    void Move(const Vector2D &mvt) {DrawableSF::Move(mvt);}

    // Render
    virtual void Render(sf::RenderTarget &App);

    // Type
    ADD_GET_TYPE_FN(OBJ_TYPE_DRAWABLE_SPRITE)
protected:

    Texture m_img;
private:
};
//------------------------------------------------------------------------------
} // namespace En

#endif // SPRITE_HPP
